/*  GREATIS FORM DESIGNER FOR .NET           */
/*  Copyright (C) 2004-2014 Greatis Software */
/*  http://www.greatis.com/dotnet/formdes/   */
/*  http://www.greatis.com/bteam.html        */

using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Text;

namespace DragDemoCS
{
   /// <summary>
   /// Show list of installed fonts.
   /// </summary>
   public class FontControl : System.Windows.Forms.UserControl
   {
      private System.Windows.Forms.NumericUpDown fontSizeUpDn;
      private System.Windows.Forms.Label label1;
      private System.Windows.Forms.CheckBox bold;
      private System.Windows.Forms.CheckBox italic;
      private DragDemoCS.FontList fonts;

      public FontControl()
      {
         InitializeComponent();

         fontSizeUpDn.ValueChanged += new EventHandler(OnFontSizeValueChanged);
         fonts.FontControl = this;
      }

      private void InitializeComponent()
      {
         this.fontSizeUpDn = new System.Windows.Forms.NumericUpDown();
         this.label1 = new System.Windows.Forms.Label();
         this.fonts = new DragDemoCS.FontList();
         this.bold = new System.Windows.Forms.CheckBox();
         this.italic = new System.Windows.Forms.CheckBox();
         ((System.ComponentModel.ISupportInitialize)(this.fontSizeUpDn)).BeginInit();
         this.SuspendLayout();
         // 
         // fontSizeUpDn
         // 
         this.fontSizeUpDn.Location = new System.Drawing.Point(88, 16);
         this.fontSizeUpDn.Name = "fontSizeUpDn";
         this.fontSizeUpDn.Size = new System.Drawing.Size(48, 20);
         this.fontSizeUpDn.TabIndex = 0;
         this.fontSizeUpDn.Value = new System.Decimal(new int[] {
                                                                   12,
                                                                   0,
                                                                   0,
                                                                   0});
         // 
         // label1
         // 
         this.label1.Location = new System.Drawing.Point(24, 18);
         this.label1.Name = "label1";
         this.label1.Size = new System.Drawing.Size(56, 16);
         this.label1.TabIndex = 1;
         this.label1.Text = "Font Size";
         // 
         // fonts
         // 
         this.fonts.BackColor = System.Drawing.Color.White;
         this.fonts.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.fonts.DrawMode = System.Windows.Forms.DrawMode.OwnerDrawVariable;
         this.fonts.Location = new System.Drawing.Point(16, 56);
         this.fonts.Name = "fonts";
         this.fonts.Size = new System.Drawing.Size(360, 320);
         this.fonts.TabIndex = 2;
         // 
         // bold
         // 
         this.bold.Location = new System.Drawing.Point(152, 16);
         this.bold.Name = "bold";
         this.bold.Size = new System.Drawing.Size(48, 24);
         this.bold.TabIndex = 3;
         this.bold.Text = "Bold";
         // 
         // italic
         // 
         this.italic.Location = new System.Drawing.Point(216, 16);
         this.italic.Name = "italic";
         this.italic.Size = new System.Drawing.Size(48, 24);
         this.italic.TabIndex = 4;
         this.italic.Text = "Italic";
         // 
         // FontControl
         // 
         this.Controls.Add(this.italic);
         this.Controls.Add(this.bold);
         this.Controls.Add(this.fonts);
         this.Controls.Add(this.label1);
         this.Controls.Add(this.fontSizeUpDn);
         this.Name = "FontControl";
         this.Size = new System.Drawing.Size(400, 384);
         ((System.ComponentModel.ISupportInitialize)(this.fontSizeUpDn)).EndInit();
         this.ResumeLayout(false);

      }

      protected override void OnSizeChanged(EventArgs e)
      {
         fonts.Location = new System.Drawing.Point(0, fonts.Location.Y);
         fonts.Size = new System.Drawing.Size(ClientSize.Width, ClientSize.Height - fonts.Location.Y);
      }


      public event EventHandler FontSizeChanged;
      
      public float FontSize
      {
         get { return (float)fontSizeUpDn.Value; }
      }

      public bool Bold
      {
         get { return bold.Checked; }
      }

      public bool Italic
      {
         get { return italic.Checked; }
      }

      private void OnFontSizeValueChanged(object sender, EventArgs e)
      {
         if( FontSizeChanged != null )
            FontSizeChanged(this, e);
      }
   }
   
   public class FontDataObject
   {
      public static readonly string format = "FontDataObject";

      public FontFamily family;
      public FontStyle  style;
      public float size;

      public FontDataObject(FontFamily family, FontStyle style, float size)
      {
         this.family = family;
         this.style = style;
         this.size = size;
      }
   }

   internal class FontMeta
   {
      public FontFamily family;
      public FontStyle style;
      
      public FontMeta(FontFamily family, FontStyle style)
      {
         this.family = family;
         this.style = style;
      }
   }

   internal class FontList : System.Windows.Forms.ListBox
   {
      int curSelected = -1;
      private InstalledFontCollection fonts;
      FontControl fontControl;

      public FontList()
      {
         this.fonts = new InstalledFontCollection();
         this.DrawMode = DrawMode.OwnerDrawVariable;

         AddItems();
      }

      public FontControl FontControl
      {
         set
         {
            if( fontControl != null )
               fontControl.FontSizeChanged -= new EventHandler(OnFontSizeChanged);

            fontControl = value;
            if( fontControl != null )
               fontControl.FontSizeChanged += new EventHandler(OnFontSizeChanged);
         }
      }

      protected override void Dispose(bool disposing)
      {
         if( fontControl != null )
            fontControl.FontSizeChanged -= new EventHandler(OnFontSizeChanged);

         base.Dispose (disposing);
      }


      void AddItems()
      {
         foreach(FontFamily family in fonts.Families)
            AddFontFamily(family);
      }

      private void AddFontFamily(FontFamily family)
      {
         if( family.IsStyleAvailable(FontStyle.Regular) )
            this.Items.Add(new FontMeta(family, FontStyle.Regular));

         /*
         if( family.IsStyleAvailable(FontStyle.Bold) )
            this.Items.Add(new FontMeta(family, FontStyle.Bold));

         if( family.IsStyleAvailable(FontStyle.Italic) )
            this.Items.Add(new FontMeta(family, FontStyle.Italic));

         if( family.IsStyleAvailable(FontStyle.Italic|FontStyle.Bold) )
            this.Items.Add(new FontMeta(family, FontStyle.Italic|FontStyle.Bold));
         */
      }

      private string FontStyleToString(FontStyle style)
      {
         if( (style & FontStyle.Bold) != 0 && (style & FontStyle.Italic) != 0 )
            return "Bold Italic";
         if( (style & FontStyle.Bold) != 0)
            return "Bold";
         if( (style & FontStyle.Bold) != 0 && (style & FontStyle.Italic) != 0 )
            return "Italic";
         return "";
      }

      void PaintItem(int index, Graphics graph)
      {
         if( this.fontControl == null )
            return;

         Graphics g = (graph != null) ? graph : Graphics.FromHwnd(Handle);

         FontMeta fm = (FontMeta)Items[index];
         Font font = new Font(fm.family, this.fontControl.FontSize, fm.style);
         
         string text = fm.family.Name + " " + FontStyleToString(fm.style);

         Brush brush = new SolidBrush(this.ForeColor);

         Color backColor = (index == curSelected) ? Color.LightSteelBlue : BackColor;
         Brush backBrush = new SolidBrush(backColor);

         Rectangle bounds = GetItemRectangle(index);
         g.FillRectangle(backBrush, bounds);
         g.DrawString(text, font, brush, bounds);
         
         backBrush.Dispose();
         brush.Dispose();
         font.Dispose();

         if( graph == null )
            g.Dispose();
      }

      protected override void OnDrawItem(DrawItemEventArgs e)
      {
         if( e.Index < 0 || e.Index >= Items.Count )
            return; 

         PaintItem(e.Index, e.Graphics);
      }

      private void ChangeSelection(int newSelected)
      {
         if( newSelected == curSelected )
            return;

         if( curSelected >= 0 )
         {
            int saveSelected = curSelected;

            curSelected = -1;
            PaintItem(saveSelected, null);
         }

         if( newSelected >= 0 )
         {
            curSelected = newSelected;
            PaintItem(curSelected, null);
         }
      }

      private int GetItemAt(Point pt)
      {
         int i = TopIndex, count = Items.Count;

         while( i < count )
         {
            Rectangle bounds = GetItemRectangle(i);

            if( bounds.Contains(pt) )
               return i;

            i++;
         }
         return -1;
      }

      protected override void OnMouseDown(MouseEventArgs e)
      {
         int index = this.GetItemAt(this.PointToClient(Control.MousePosition));
         if( index >= 0 )
            ChangeSelection(index);

         base.OnMouseDown (e);
      }

      protected override void OnMouseMove(MouseEventArgs e)
      {
         int index = GetItemAt(PointToClient(Control.MousePosition));
         if( (Control.MouseButtons & MouseButtons.Left) == 0 || index < 0 )
         {
            base.OnMouseMove (e);
            return;
         }

         FontMeta fm = (FontMeta)Items[index];
         FontStyle fs = fm.style;

         if( fontControl.Bold )
            fs |= FontStyle.Bold;
         if( fontControl.Italic )
            fs |= FontStyle.Italic;

         IDataObject data = new DataObject(FontDataObject.format, 
            new FontDataObject(fm.family, fs, this.fontControl.FontSize));

         this.DoDragDrop(data, DragDropEffects.Copy);
      }


      private int CalcItemHeight(int item)
      {
         FontMeta fm = (FontMeta)Items[item];
         if( fm == null || fontControl == null )
            return ItemHeight;

         Font font = new Font(fm.family, this.fontControl.FontSize, fm.style);
         Graphics g = Graphics.FromHwnd(this.Handle);

         int itemHeight = (int)Math.Floor(g.MeasureString("Q", font).Height);

         font.Dispose();
         g.Dispose();

         return itemHeight;
      }

      protected override void OnMeasureItem(MeasureItemEventArgs e)
      {
         if( e.Index < 0 || e.Index >= Items.Count )
            return; 

         e.ItemWidth = this.ClientRectangle.Width;
         e.ItemHeight = CalcItemHeight(e.Index);
      }


      private void OnFontSizeChanged(object sender, EventArgs e)
      {
         int topIndex = this.TopIndex;

         this.BeginUpdate();
         this.Items.Clear();

         AddItems();
         this.TopIndex = topIndex;

         this.EndUpdate();
      }
   }
}
