/*  GREATIS FORM DESIGNER FOR .NET           */
/*  Copyright (C) 2004-2014 Greatis Software */
/*  http://www.greatis.com/dotnet/formdes/   */
/*  http://www.greatis.com/bteam.html        */

using System;
using System.Drawing;
using System.Drawing.Design;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Data;

namespace DragDemoCS
{
   /// <summary>
   /// Summary description for Form1.
   /// </summary>
   public class FormMain : System.Windows.Forms.Form
   {
      private Greatis.FormDesigner.Designer designer1;
      private Greatis.FormDesigner.Treasury treasury1;
      private System.Windows.Forms.Button design;

      private ToolBoxForm toolbox;
      private PropertyWindow properties;
      private System.Windows.Forms.Form fonts;
      
      /// <summary>
      /// Required designer variable.
      /// </summary>
      private System.ComponentModel.Container components = null;

      public FormMain()
      {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         //
         // TODO: Add any constructor code after InitializeComponent call
         //
         toolbox = new ToolBoxForm();
         fonts = new FontForm();
         properties = new PropertyWindow();

         toolbox.ToolboxControl.Designer = designer1;
         properties.Designer = designer1;

         InitToolbox();

         // register text handler
         designer1.RegisterDragAndDropHandler(DataFormats.Text, 
            new Greatis.FormDesigner.CreateComponentsCallback(CreateTextElement));

         // register font changes handler 
         designer1.RegisterDragAndDropHandler(FontDataObject.format, 
            new Greatis.FormDesigner.CreateComponentsCallback(SetFont));
      }

      public IComponent[] SetFont(string format, object serializedObject, IDesignerHost host)
      {
         FontDataObject fontObject = (FontDataObject)((IDataObject)serializedObject).GetData(format);

         if (fontObject != null)
         {
            Control c = host.RootComponent as Control;
            Control selectedControl = c.GetChildAtPoint(c.PointToClient(Control.MousePosition));

            if (selectedControl != null)
            {
               Font f = new Font(fontObject.family, fontObject.size, fontObject.style);
               selectedControl.Font = f;
            }
         }
         return new IComponent[] { };
      }

      delegate void SetTextMethodHandler(Control c, string text);
      public IComponent[] CreateTextElement(string format, object serializedObject, IDesignerHost host)
      {
         IDataObject dataObject = serializedObject as IDataObject;
         System.Windows.Forms.Control textbox = (Control)host.CreateComponent(typeof(Label));
         string text = (string)dataObject.GetData(DataFormats.Text);

         // Because the designer resets the text of the textbox, use 
         // a SetTextMethodHandler to set the text to the value of 
         // the text data.
         Control c = host.RootComponent as Control;
         c.BeginInvoke(new SetTextMethodHandler(OnSetText), new object[] { textbox, text });

         return new IComponent[] { textbox };
      }

      // Method to set the text property of a TextBox after it is initialized.
      private void OnSetText(Control c, string text)
      {
         c.Text = text;
      }

      void InitToolbox()
      {
         IToolboxService itbs = (IToolboxService)designer1.DesignerHost.GetService(typeof(IToolboxService));

         itbs.AddToolboxItem(new System.Drawing.Design.ToolboxItem(typeof(Label)));
         itbs.AddToolboxItem(new System.Drawing.Design.ToolboxItem(typeof(TextBox)));
         itbs.AddToolboxItem(new System.Drawing.Design.ToolboxItem(typeof(TabControl)));
         itbs.AddToolboxItem(new System.Drawing.Design.ToolboxItem(typeof(Panel)));

         itbs.AddToolboxItem(new System.Drawing.Design.ToolboxItem(typeof(TreeView)), "Additional");
         itbs.AddToolboxItem(new System.Drawing.Design.ToolboxItem(typeof(ListView)), "Additional");

         itbs.AddToolboxItem(new System.Drawing.Design.ToolboxItem(typeof(Timer)), "Components");
      }

      void StartDesign()
      {
         this.design.Text = "End design";

         if( designer1.DesignedForm == null )
         {
            Form designedForm = new Form();
            designedForm.Location = new Point(this.Left, this.Bottom);
            designedForm.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
            designedForm.Text = "Designed Form";

            designedForm.Show();

            designer1.DesignedForm = designedForm;
         }

         properties.Location = new Point(designer1.DesignedForm.Right, 10);
         properties.Show();
         properties.Closing += new CancelEventHandler(OnFormClosing);

         toolbox.Location = new Point(properties.Location.X, properties.Bottom);
         toolbox.Size = new Size(properties.Width, toolbox.Height);
         toolbox.Show();
         toolbox.Closing += new CancelEventHandler(OnFormClosing);

         fonts.Location = new Point(toolbox.Location.X, toolbox.Bottom);
         fonts.Size = new Size(properties.Width, fonts.Height);
         fonts.Show();
         fonts.Closing += new CancelEventHandler(OnFormClosing);

         designer1.Active = true;
      }

      void EndDesign()
      {
         this.design.Text = "Start design";

         designer1.Active = false;

         properties.Hide();
         properties.Closing -= new CancelEventHandler(OnFormClosing);

         toolbox.Hide();
         toolbox.Closing -= new CancelEventHandler(OnFormClosing);

         fonts.Hide();
         fonts.Closing -= new CancelEventHandler(OnFormClosing);

      }

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing )
      {
         if( disposing )
         {
            if (components != null) 
            {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

      #region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent()
      {
			System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(FormMain));
			this.designer1 = new Greatis.FormDesigner.Designer();
			this.treasury1 = new Greatis.FormDesigner.Treasury();
			this.design = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// designer1
			// 
			this.designer1.Active = false;
			this.designer1.DesignContainer = null;
			this.designer1.DesignedComponents = null;
			this.designer1.DesignedForm = null;
			this.designer1.FormTreasury = this.treasury1;
			this.designer1.GridSize = new System.Drawing.Size(8, 8);
			this.designer1.LogFile = null;
			this.designer1.ShowErrorMessage = true;
			// 
			// treasury1
			// 
			this.treasury1.Components = null;
			this.treasury1.DesignerHost = null;
			this.treasury1.LogFile = null;
			this.treasury1.ShowErrorMessage = true;
			// 
			// design
			// 
			this.design.Location = new System.Drawing.Point(84, 49);
			this.design.Name = "design";
			this.design.Size = new System.Drawing.Size(120, 32);
			this.design.TabIndex = 0;
			this.design.Text = "Start Design";
			this.design.Click += new System.EventHandler(this.design_Click);
			// 
			// FormMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(284, 134);
			this.Controls.Add(this.design);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "FormMain";
			this.Text = "Greatis Form Designer .Net  |  Drag Demo";
			this.ResumeLayout(false);

      }
      #endregion

      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      [STAThread]
      static void Main() 
      {
         Application.Run(new FormMain());
      }

      private void design_Click(object sender, System.EventArgs e)
      {
         if( designer1.Active )
            EndDesign();
         else
            StartDesign();
      }

      private void OnFormClosing(object sender, CancelEventArgs e)
      {
         ((Form)sender).Hide();
         e.Cancel = true;
      }
   }
}
